@@ -3,7 +3,8 @@ from typing import Union, Literal, Callable
 from pydna.dseqrecord import Dseqrecord
 from pydna.primer import Primer as PydnaPrimer
 from pydna.crispr import cas9
-from pydantic import conlist, create_model
+from pydantic import create_model, Field
+from typing import Annotated
 from Bio.Restriction.Restriction import RestrictionBatch
 from opencloning.cre_lox import cre_loxP_overlap, annotate_loxP_sites
 from ..dna_functions import (
@@ -27,7 +28,7 @@ from ..pydantic_models import (
     CreLoxRecombinationSource,
     InVivoAssemblySource,
 )
-from ..assembly2 import (
+from pydna.assembly2 import (
     Assembly,
     assemble,
     sticky_end_sub_strings,
@@ -39,6 +40,7 @@ from ..assembly2 import (
     blunt_overlap,
     combine_algorithms,
     annotate_primer_binding_sites,
+    common_sub_strings,
 )
 
 from ..gateway import gateway_overlap, find_gateway_sites, annotate_gateway_sites
@@ -79,8 +81,8 @@ def format_known_assembly_response(
 )
 async def crispr(
     source: CRISPRSource,
-    guides: list[PrimerModel],
-    sequences: conlist(TextFileSequence, min_length=2, max_length=2),
+    guides: Annotated[list[PrimerModel], Field(min_length=1)],
+    sequences: Annotated[list[TextFileSequence], Field(min_length=2, max_length=2)],
     minimal_homology: int = Query(40, description='The minimum homology between the template and the insert.'),
 ):
     """Return the sequence after performing CRISPR editing by Homology directed repair
@@ -89,6 +91,9 @@ async def crispr(
     """
     template, insert = [read_dsrecord_from_json(seq) for seq in sequences]
 
+    if template.circular:
+        raise HTTPException(400, 'Circular DNA targets are not supported for CRISPR editing.')
+
     # TODO: check input method for guide (currently as a primer)
     # TODO: support user input PAM
 
@@ -102,6 +107,7 @@ async def crispr(
                 400, f'Could not find Cas9 cutsite in the target sequence using the guide: {guide.name}'
             )
         guide_cuts.append(possible_cuts)
+    sorted_guide_ids = list(sorted([guide.id for guide in guides]))
 
     # Check if homologous recombination is possible
     fragments = [template, insert]
@@ -140,16 +146,17 @@ async def crispr(
     # meant for linear DNA
 
     out_sources = [
-        CRISPRSource.from_assembly(id=source.id, assembly=a, guides=source.guides, fragments=fragments)
+        CRISPRSource.from_assembly(id=source.id, assembly=a, guides=sorted_guide_ids, fragments=fragments)
         for a in valid_assemblies
     ]
 
     # If a specific assembly is requested
-    if len(source.assembly):
+    if source.is_assembly_complete():
         return format_known_assembly_response(source, out_sources, [template, insert])
 
     out_sequences = [
-        format_sequence_genbank(assemble([template, insert], a), source.output_name) for a in valid_assemblies
+        format_sequence_genbank(assemble([template, insert], a, is_insertion=True), source.output_name)
+        for a in valid_assemblies
     ]
     return {'sources': out_sources, 'sequences': out_sequences}
 
@@ -199,7 +206,7 @@ def generate_assemblies(
         raise HTTPException(400, *e.args)
 
     # If a specific assembly is requested
-    if len(source.assembly):
+    if source.is_assembly_complete():
         return format_known_assembly_response(source, out_sources, fragments, product_callback)
 
     out_sequences = [
@@ -220,7 +227,7 @@ def generate_assemblies(
 )
 async def ligation(
     source: LigationSource,
-    sequences: conlist(TextFileSequence, min_length=1),
+    sequences: Annotated[list[TextFileSequence], Field(min_length=1)],
     blunt: bool = Query(False, description='Use blunt ligation as well as sticky ends.'),
     allow_partial_overlap: bool = Query(False, description='Allow for partially overlapping sticky ends.'),
     circular_only: bool = Query(False, description='Only return circular assemblies.'),
@@ -234,7 +241,7 @@ async def ligation(
 
     # If the assembly is known, the blunt parameter is ignored, and we set the algorithm type from the assembly
     # (blunt ligations have features without length)
-    if len(source.assembly):
+    if source.is_assembly_complete():
         asm = source.get_assembly_plan(fragments)
         blunt = len(asm[0][2]) == 0
 
@@ -256,14 +263,18 @@ async def ligation(
 )
 async def pcr(
     source: PCRSource,
-    sequences: conlist(TextFileSequence, min_length=1, max_length=1),
-    primers: conlist(PrimerModel, min_length=1, max_length=2),
-    minimal_annealing: int = Query(20, description='The minimal annealing length for each primer.'),
+    sequences: Annotated[list[TextFileSequence], Field(min_length=1, max_length=1)],
+    primers: Annotated[list[PrimerModel], Field(min_length=1, max_length=2)],
+    minimal_annealing: int = Query(
+        14,
+        description='The minimal amount of bases that must match between the primer and the sequence, excluding mismatches.',
+    ),
     allowed_mismatches: int = Query(0, description='The number of mismatches allowed'),
 ):
     if len(primers) != len(sequences) * 2:
         raise HTTPException(400, 'The number of primers should be twice the number of sequences.')
 
+    minimal_annealing = minimal_annealing + allowed_mismatches
     pydna_sequences = [read_dsrecord_from_json(s) for s in sequences]
     pydna_primers = [PydnaPrimer(p.sequence, id=str(p.id), name=p.name) for p in primers]
 
@@ -272,7 +283,7 @@ async def pcr(
     # What happens if annealing is zero? That would mean
     # mismatch in the 3' of the primer, which maybe should
     # not be allowed.
-    if len(source.assembly):
+    if source.is_assembly_complete():
         minimal_annealing = source.minimal_overlap()
         # Only the ones that match are included in the output assembly
         # location, so the submitted assembly should be returned without
@@ -310,11 +321,11 @@ async def pcr(
     ]
 
     # If a specific assembly is requested
-    if len(source.assembly):
+    if source.is_assembly_complete():
 
         def callback(x):
             if source.add_primer_features:
-                return annotate_primer_binding_sites(x, fragments, source.get_assembly_plan(fragments))
+                return annotate_primer_binding_sites(x, fragments)
             else:
                 return x
 
@@ -326,7 +337,7 @@ async def pcr(
     def callback(fragments, a):
         out_seq = assemble(fragments, a)
         if source.add_primer_features:
-            return annotate_primer_binding_sites(out_seq, fragments, possible_assemblies)
+            return annotate_primer_binding_sites(out_seq, fragments)
         else:
             return out_seq
 
@@ -348,14 +359,14 @@ async def pcr(
 )
 async def homologous_recombination(
     source: HomologousRecombinationSource,
-    sequences: conlist(TextFileSequence, min_length=2, max_length=2),
+    sequences: Annotated[list[TextFileSequence], Field(min_length=2, max_length=2)],
     minimal_homology: int = Query(40, description='The minimum homology between the template and the insert.'),
 ):
 
     template, insert = [read_dsrecord_from_json(seq) for seq in sequences]
 
     # If an assembly is provided, we ignore minimal_homology
-    if len(source.assembly):
+    if source.is_assembly_complete():
         minimal_homology = source.minimal_overlap()
 
     asm = Assembly((template, insert), limit=minimal_homology, use_all_fragments=True)
@@ -381,11 +392,14 @@ async def homologous_recombination(
     ]
 
     # If a specific assembly is requested
-    if len(source.assembly):
+    if source.is_assembly_complete():
         return format_known_assembly_response(source, out_sources, [template, insert])
 
     out_sequences = [
-        format_sequence_genbank(assemble([template, insert], a), source.output_name) for a in possible_assemblies
+        format_sequence_genbank(
+            assemble([template, insert], a, is_insertion=not template.circular), source.output_name
+        )
+        for a in possible_assemblies
     ]
 
     return {'sources': out_sources, 'sequences': out_sequences}
@@ -403,7 +417,7 @@ async def homologous_recombination(
     ),
 )
 async def gibson_assembly(
-    sequences: conlist(TextFileSequence, min_length=1),
+    sequences: Annotated[list[TextFileSequence], Field(min_length=1)],
     source: Union[GibsonAssemblySource, OverlapExtensionPCRLigationSource, InFusionSource, InVivoAssemblySource],
     minimal_homology: int = Query(
         40, description='The minimum homology between consecutive fragments in the assembly.'
@@ -417,8 +431,9 @@ async def gibson_assembly(
     def create_source(a, is_circular):
         return source.__class__.from_assembly(assembly=a, circular=is_circular, id=source.id, fragments=fragments)
 
+    algo = gibson_overlap if not isinstance(source, InVivoAssemblySource) else common_sub_strings
     resp = generate_assemblies(
-        source, create_source, fragments, circular_only, gibson_overlap, False, {'limit': minimal_homology}
+        source, create_source, fragments, circular_only, algo, False, {'limit': minimal_homology}
     )
 
     if len(resp['sources']) == 0:
@@ -441,7 +456,7 @@ async def gibson_assembly(
 )
 async def restriction_and_ligation(
     source: RestrictionAndLigationSource,
-    sequences: conlist(TextFileSequence, min_length=1),
+    sequences: Annotated[list[TextFileSequence], Field(min_length=1)],
     allow_partial_overlap: bool = Query(False, description='Allow for partially overlapping sticky ends.'),
     circular_only: bool = Query(False, description='Only return circular assemblies.'),
 ):
@@ -483,7 +498,7 @@ async def restriction_and_ligation(
 )
 async def gateway(
     source: GatewaySource,
-    sequences: conlist(TextFileSequence, min_length=1),
+    sequences: Annotated[list[TextFileSequence], Field(min_length=1)],
     circular_only: bool = Query(False, description='Only return circular assemblies.'),
     only_multi_site: bool = Query(
         False, description='Only return assemblies where more than one site per sequence recombined.'
@@ -528,7 +543,7 @@ async def gateway(
         multi_site_sources = [
             i
             for i, s in enumerate(resp['sources'])
-            if all(join.left_location != join.right_location for join in s.assembly)
+            if all(join.left_location != join.right_location for join in s.input)
         ]
         sources = [resp['sources'][i] for i in multi_site_sources]
         sequences = [resp['sequences'][i] for i in multi_site_sources]
@@ -545,7 +560,9 @@ async def gateway(
         sequences=(list[TextFileSequence], ...),
     ),
 )
-async def cre_lox_recombination(source: CreLoxRecombinationSource, sequences: conlist(TextFileSequence, min_length=1)):
+async def cre_lox_recombination(
+    source: CreLoxRecombinationSource, sequences: Annotated[list[TextFileSequence], Field(min_length=1)]
+):
     fragments = [read_dsrecord_from_json(seq) for seq in sequences]
 
     # Lambda function for code clarity
